# BaseGraph

How do we "load" a map into GerryChain? We start by initializing a BaseGraph
object from either a .json or .shp file.

## Properties
Once you initialize a `BaseGraph`, you can access the following properties from it:

| BaseGraph Properties                       | Description                                                                              |
|--------------------------------------------|------------------------------------------------------------------------------------------|
| `num_nodes` (Int)                          | Number of nodes in the BaseGraph                                                         |
| `num_edges` (Int)                          | Number of edges in the BaseGraph                                                         |
| `total_pop` (Int)                          | Total Population in the BaseGraph                                                        |
| `populations` Array{Int, 1}                | An array of populations at the node level, where the i'th index of the array is the population of the i'th node |
| `adj_matrix` SparseMatrixCSC{Int, Int}     | A sparse adjacency matrix of the graph, where an edge exists between nodes `i` and `j` if `adj_matrix[i,j] != 0`. The value at `adj_matrix[i,j]` is the edge id of the edge that connects nodes `i` and `j`. |
| `edge_src` Array{Int, 1}                   | An array of length(num_edges) where `edge_src[i]` is the source of the edge `i`. |
| `edge_dst` Array{Int, 1}                   | An array of length(num_edges) where `edge_dst[i]` is the destination of the edge `i`. |
| `neighbors` Array{Array{Int64,1},1}        | An array of arrays, where `neighbors[i]` holds a list of all the neighbors of node `i` |
| `simple_graph` SimpleGraph                 | A SimpleGraph object from the LightGraphs library |
| `attributes` Array{Dict{String, Any}}      | An array of dictionaries where `attributes[i]` holds the attributes of node `i` |

*BaseGraph*

```@index
Order = [:type, :function]
Pages   = ["graph.md"]
```

## Initializing a BaseGraph
A BaseGraph can be initialized in the following way:

```@docs
BaseGraph
```

If you initialize a BaseGraph from a .shp file, make sure that there is
another file of the same name with a .dbf extension in the same folder!
This is because the .shp file provides the geometry of the regions of interest,
while the .dbf provides information about each region's attributes - e.g.,
the total number of votes cast or the percentage of Black voters.

If you initialize a BaseGraph from a .json file, GerryChain expects the .json file
to be generated by the Graph.to_json() function of the Python implementation of GerryChain.
We assume that the JSON file has the structure of a dictionary where

(1) the key "nodes" yields an array of dictionaries of node attributes,

(2) the key "adjacency" yields an array of edges (represented as dictionaries), and

(3) the key "id" within the edge dictionary indicates the destination node of the edge.


## API

```@autodocs
Modules = [GerryChain]
Pages   = ["graph.jl"]
Private = false
Filter = t -> t != BaseGraph
```
