# Data visualization

After running a chain, it's common to want to create some data visualizations that summarize various statistics of interest across plans in the chain (e.g., a compactness score, party vote share by district, etc.). As part of the GerryChainJulia library, we use [PyPlot.jl](https://github.com/JuliaPy/PyPlot.jl) to create some functions that will generate some common types of graphs.

```@index
Order = [:type, :function]
Pages   = ["plotting.md"]
```

## score_boxplot()

Boxplots can be helpful for visualizing the "typical" range of values for a statistic of interest for all the plans in the chain. For example, users interested to know whether "cracking and packing" has occurred might compare the percentage of minority voters in each district in an enacted plan overlaid on a set of boxplots representing the typical range of percentage of minority voters for plans generated by the chain. Users can generate boxplots for any score of interest. If the score is a district-wide score (e.g., percentage of white voters in each district), then there will be multiple boxplots shown in the same figure, with each boxplot representing the range of scores for a particular district. If the score is a plan-wide score, then there will be one boxplot in the figure.

```@docs
score_boxplot
```

**Usage**
```
# run chain
chain_data = recom_chain(...)

# graph results and compare to enacted plan!
# the length of `plan1_dshare` and `plan2_dshare` should be equal to the total # of districts
plan1_dshare = [...]
plan2_dshare = [...]
score_boxplot(chain_data, "dem_vote_share", comparison_scores=[("plan1", plan1_dshare), ("plan2", plan2_dshare)])
# without comparison scores:
# score_boxplot(chain_data, "dem_vote_share")

# if you want to edit anything about the default plot, you can simply use plt
plt.ylabel("Democratic vote share")
```
**Example of generated plot**

![boxplot](https://user-images.githubusercontent.com/5581093/87839451-b56ccf00-c84f-11ea-92c1-83581efa29fa.png)

## score_histogram()

This function allows users to easily create histogram graphs of a (plan-level) score of interest (e.g., the number of cut edges, the number of seats won by a particular party, etc.) (Unlike the `score_boxplot` function, this function cannot be used on district-level scores.) Similar to `score_boxplot`, you can also pass in "comparison scores" to visualize where a particular value of a score lies in relation to the histogram of values observed during the process of the chain.

```@docs
score_histogram
```                             

**Usage**
```
# run chain
chain_data = recom_chain(...)

# graph results and compare to enacted plan!
score_histogram(chain_data, "cut_edges", comparison_scores=[ ("enacted", 21) ]) # if the enacted plan has 21 cut edges
# score_histogram(chain_data, "cut_edges", comparison_scores=[ ("enacted", 21) ], bins=3, rwidth=1) # we also support passing in a few arguments that can be passed into matplotlib
# score_histogram(chain_data, "cut_edges") # without any comparison scores

# if user wants to edit anything about the default plot, they can simply use plt
plt.xlabel("efficiency_gap")
```
**Example of generated histogram**

![boxplot](https://user-images.githubusercontent.com/5581093/87839677-cc5ff100-c850-11ea-84a0-af06fb215a38.png)
